//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


public class DataDirectoryPathFileWriter {
	private File statusFile;

	private static final String TEMPORARY_FILE_SUFFIX = ".tmp";

	public DataDirectoryPathFileWriter(File file) {
		statusFile = file;
	}

	public DataDirectoryPathFileWriter(String filePath) {
		this(new File(filePath));
	}

	public void writeFile(File dataDirectory)
			throws IOException {
		writeFile(dataDirectory.getAbsolutePath());
	}

	public void writeFile(String dataDirectoryPath)
			throws IOException {
		File temporaryFile = new File(statusFile.getAbsolutePath() +
				TEMPORARY_FILE_SUFFIX);
		temporaryFile.delete();

		BufferedWriter writer = null;

		try {
			writer = new BufferedWriter(new FileWriter(temporaryFile));
			if (dataDirectoryPath != null)
				writer.write(dataDirectoryPath);
			writer.close();
			temporaryFile.renameTo(statusFile);
		} finally {
			if (writer != null)
				writer.close();
		}
	}
}