//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

public class StatusFileReader {
	private String lastGame;
	private Hashtable<String, Hashtable<String, Boolean>> expansionChecks;

	public StatusFileReader(File file)
			throws IOException, FileNotFoundException {
		lastGame = null;
		expansionChecks = new Hashtable<String, Hashtable<String, Boolean>>();
		readFile(file);
	}

	public StatusFileReader(String filePath)
			throws IOException, FileNotFoundException {
		this(new File(filePath));
	}

	public String getLastGame() {
		return lastGame;
	}

	public boolean getExpansionCheck(String gameId, String expansionId) {
		Hashtable<String, Boolean> expansions = expansionChecks.get(gameId);
		if (expansions == null)
			return false;
		Boolean used = expansions.get(expansionId);
		if (used == null)
			return false;
		return used;
	}

	private void readFile(File file)
			throws IOException, FileNotFoundException {
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(file));
			String line;
			while ((line = reader.readLine()) != null)
				parseLine(line);
		} finally {
			reader.close();			
		}
	}

	private void parseLine(String line) {
		List<String> fields = splitLineIntoFields(line);
		int size = fields.size();
		if (size == 0)
			return;

		String command = fields.get(0);
		if (command.equals("last-game") && size == 2)
			lastGame = fields.get(1);
		else if (command.equals("expansion-check") && size == 4) {
			Hashtable<String, Boolean> dic =
					expansionChecks.get(fields.get(1));
			if (dic == null)
				dic = new Hashtable<String, Boolean>();
			dic.put(fields.get(2), Boolean.parseBoolean(fields.get(3)));
			expansionChecks.put(fields.get(1), dic);
		}
	}

	private List<String> splitLineIntoFields(String line) {
		List<String> list = new ArrayList<String>();
		String ln = line.trim();
		for (;;) {
			int length = ln.length();
			int index = ln.indexOf('\t');
			if (index < 0) {
				list.add(unescapeString(ln));
				return list;
			}
			list.add(unescapeString(ln.substring(0, index)));
			ln = ln.substring(index + 1, length);
		}
	}

	private String unescapeString(String string) {
		StringBuffer strIn  = new StringBuffer(string);
		StringBuffer strOut = new StringBuffer();
		int lengthIn = strIn.length();

		int i = 0;
		while (i < lengthIn) {
			int c = strIn.codePointAt(i);
			if (c == '%' && i + 2 < lengthIn) {
				int hex1 = strIn.codePointAt(i + 1);
				int hex2 = strIn.codePointAt(i + 2);
				int c2 = 0;
				if ('0' <= hex1 && hex1 <= '9')
					c2 = (hex1 - '0') << 4;
				else if ('A' <= hex1 && hex1 <= 'F')
					c2 = (hex1 - 'A' + 0x0a) << 4;
				else if ('a' <= hex1 && hex1 <= 'f')
					c2 = (hex1 - 'a' + 0x0a) << 4;
				else {
					strOut.appendCodePoint(c);
					i++;
					continue;
				}
				if ('0' <= hex2 && hex2 <= '9')
					c2 |= hex2 - '0';
				else if ('A' <= hex2 && hex2 <= 'F')
					c2 |= hex2 - 'A' + 0x0a;
				else if ('a' <= hex2 && hex2 <= 'f')
					c2 |= hex2 - 'a' + 0x0a;
				else {
					strOut.appendCodePoint(c);
					i++;
					continue;
				}
				strOut.appendCodePoint(c2);
				i += 3;
			} else {
				strOut.appendCodePoint(c);
				i++;
			}
		}

		return strOut.toString();
	}
}
