//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;


public class StatusFileWriter {
	private File statusFile;
	private String lastGame;
	private Hashtable<String, Hashtable<String, Boolean>> expansionChecks;

	private static final String TEMPORARY_FILE_SUFFIX = ".tmp";

	public StatusFileWriter(File file) {
		statusFile = file;
		lastGame = null;
		expansionChecks = new Hashtable<String, Hashtable<String, Boolean>>();
	}

	public StatusFileWriter(String filePath) {
		this(new File(filePath));
	}

	public void setLastGame(String gameId) {
		lastGame = gameId;
	}

	public void setExpansionCheck(String gameId, String expansionId,
			boolean used) {
		Hashtable<String, Boolean> dic = expansionChecks.get(gameId);
		if (dic == null) {
			dic = new Hashtable<String, Boolean>();
			expansionChecks.put(gameId, dic);
		}
		dic.put(expansionId, (Boolean)used);
	}

	public void writeFile()
			throws IOException {
		File temporaryFile = new File(statusFile.getAbsolutePath() +
				TEMPORARY_FILE_SUFFIX);
		temporaryFile.delete();

		BufferedWriter writer = null;

		try {
			writer = new BufferedWriter(new FileWriter(temporaryFile));
			if (lastGame != null) {
				writer.write("last-game\t");
				writer.write(escapeString(lastGame));
				writer.newLine();
			}
			Enumeration<String> en = expansionChecks.keys();
			while (en.hasMoreElements()) {
				String game = en.nextElement();
				Hashtable<String, Boolean> dic = expansionChecks.get(game);
				Enumeration<String> en2 = dic.keys();
				while (en2.hasMoreElements()) {
					String expansion = en2.nextElement();
					Boolean value = dic.get(expansion);
					writer.write("expansion-check\t");
					writer.write(escapeString(game));
					writer.write('\t');
					writer.write(escapeString(expansion));
					writer.write('\t');
					writer.write(value.toString());
					writer.newLine();
				}
			}
			writer.close();
		} finally {
			if (writer != null)
				writer.close();
		}

		temporaryFile.renameTo(statusFile);
	}

	private String escapeString(String string) {
		StringBuffer strIn  = new StringBuffer(string);
		StringBuffer strOut = new StringBuffer();
		int lengthIn = strIn.length();

		for (int i = 0; i < lengthIn; i++) {
			int c = strIn.codePointAt(i);
			if (c < ' ' || c == '%') {
				strOut.append('%');
				int hex1 = (c & 0xf0) >> 4;
			int hex2 = (c & 0x0f);
			if (hex1 <= 9)
				strOut.appendCodePoint('0' + hex1);
			else
				strOut.appendCodePoint('a' + hex1 - 10);
			if (hex2 <= 9)
				strOut.appendCodePoint('0' + hex2);
			else
				strOut.appendCodePoint('a' + hex2 - 10);
			} else {
				strOut.appendCodePoint(c);
			}
		}

		return strOut.toString();
	}
}
