#! /bin/sh
#
# Copyright (c) 2013  Motoyuki Kasahara
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
######################################################################
#
# NAME
#     src-archive.sh - create a source distribution file for GameRandomizer
#
# SYNPOSIS
#     src-archive.sh
#
# DESCRIPTION
#     'src-archive.sh' creates "gamerandomizer-src-<VERSION>.tar.gz" file.
#     VERSION is gotten from 'GameRandomizer/AndroidManifest.xml' file.
#
PACKAGE=gamerandomizer-src
VERSION=`sed -n -e 's/^.*android:versionName="\([0-9.]*\)".*$/\1/p' \
	GameRandomizer/AndroidManifest.xml`

PACKAGE_BASENAME="${PACKAGE}-${VERSION}"
rm -rf $PACKAGE_BASENAME
mkdir -p $PACKAGE_BASENAME

cat DISTFILES | while read PATTERN; do
	ls -1 $PATTERN | while read FILEPATH; do
		[ -f $FILEPATH ] || continue
		DISTDIR="$PACKAGE_BASENAME/`dirname $FILEPATH`"
		DISTFILE="`basename $FILEPATH`"
		mkdir -p $DISTDIR
		if [ $? -ne 0 ]; then
			echo "$0: failed to create the directory: $DISTDIR" 1>&2
			exit 1
		fi
		cp -p $FILEPATH $DISTDIR/$DISTFILE
		echo "archive: $FILEPATH"
	done
done

export GZIP=-9
PACKAGE_FILE="$PACKAGE_BASENAME.tar.gz"
echo "create: $PACKAGE_FILE"
rm -f $PACKAGE_FILE
tar -c -z -f $PACKAGE_FILE $PACKAGE_BASENAME
rm -rf $PACKAGE_BASENAME
echo "done"

exit 0
